# Generated by roxygen2: do not edit by hand

S3method(predict_savedmodel,export_prediction)
S3method(predict_savedmodel,graph_prediction)
S3method(predict_savedmodel,webapi_prediction)
S3method(print,savedmodel_predictions)
export(export_savedmodel)
export(load_savedmodel)
export(predict_savedmodel)
export(serve_savedmodel)
export(view_savedmodel)
import(jsonlite)
import(swagger)
import(tensorflow)
importFrom(httpuv,runServer)
importFrom(magrittr,"%>%")
importFrom(reticulate,import_builtins)
importFrom(tensorflow,export_savedmodel)
importFrom(tensorflow,view_savedmodel)
importFrom(tools,file_ext)
importFrom(utils,untar)
